package day2

import java.nio.file.Files
import java.nio.file.Path

class Day2 {
    private lateinit var initialList: List<Int>
    private lateinit var currentList: MutableList<Int>

    fun runPart1(inputFile: Path): Int {
        loadFile(inputFile)
        return solve()
    }


    fun runPart2(inputFile: Path): Int {
        loadFile(inputFile)
        for (noun in 0..99) {
            for (verb in 0..99) {
                currentList[1] = noun
                currentList[2] = verb
                with(solve()) {
                    if (this == 19690720) {
                        return (100 * noun) + verb
                    }
                    else reset()
                }
            }
        }
        throw RuntimeException("Can't be found")
    }

    private fun solve(): Int {
        var index = 0
        while ( true ) {
            when (currentList[index]) {
                1 -> {
                    val firstCoord = currentList[index + 1]
                    val secondCoord = currentList[index + 2]
                    val outputCoord = currentList[index + 3]
                    applyOperation(firstCoord, secondCoord, outputCoord) { a: Int, b: Int -> a + b }
                }
                2 -> {
                    val firstCoord = currentList[index + 1]
                    val secondCoord = currentList[index + 2]
                    val outputCoord = currentList[index + 3]
                    applyOperation(firstCoord, secondCoord, outputCoord) { a: Int, b: Int -> a * b }
                }
                99 -> {
                    return currentList[0]
                }
                else -> throw RuntimeException("Error encountered")
            }
            index += 4
        }
    }
    private fun reset(): Unit {
        currentList = initialList.toMutableList()
        println(currentList)
    }

    private fun applyOperation(
        firstCoord: Int,
        secondCoord: Int,
        outputCoord: Int,
        operation: (Int, Int) -> Int
    ): Unit {
        currentList[outputCoord] = operation(currentList[firstCoord], currentList[secondCoord])
    }

    private fun loadFile(inputFile: Path): Unit {
        initialList = Files.newBufferedReader(inputFile).readLine().split(',').map { it.toInt() }
        currentList = initialList.toMutableList()
    }
}